#include <math.h>
#include <stdio.h>

extern int dsyevv3(double A[3][3], double Q[3][3], double w[3]);
/**
# funcion to calculate the real roots
# of a cubic equation
# 
# It based on 
# A new approach to solving the cubic Cardan's
# solution revealed
# R.W.D. Nickalls
# The mathematical gazette, 1993, vol 77, pp. 354-359
#
# Fraga  Jun 6, 2010
 C version, Sep 21, 2011
**/

double cubic_root( double v ) 
{
	double signo = 1.0;
	double del;

	if( v < 0.0 ) {
		signo = -1.0;
		v *= -1.0;
	}
	del = signo*exp( log(v)/3.0 );

	return( del );
}

/** 
 Args: ax^3 + bx^2 + cx + d = 0
 double * roots must be pointer to an array of size three 
 Returns the number of real root and their values insise roots[]
**/

int cubic( double a, double b, double c, double d, double *roots )
{
	double xN, yN, yN2;
	double delta2, h2, delta;

	xN = -b/(3.0*a);

	yN = xN*( xN*(a*xN + b) + c ) + d;
	yN2 = yN*yN;

	delta2 = (b*b - 3.0*a*c)/(9.0*a*a); 
	h2 = 4.0*a*a*delta2*delta2*delta2;

	if ( fabs(yN2 - h2) < 1e-8 ) {
		if ( fabs( yN2 ) < 1e-8 ){
			// Three equal roots in xN 
			roots[0] = roots[1] = roots[2] = xN;
			return( 3 );
		}
		else {
			// print "Dos reales iguales y la tercera distinta\n";
			delta = cubic_root( yN/(2.0*a) );
			roots[0] = xN + delta;
			roots[1] = roots[2] = xN - 2.0*delta;
			return( 3 );
		}
	}
	else if ( yN2 > h2 ) {
		double v, v1, v2;

		// printf( "Una raiz real\n" );
		v = sqrt( yN2 - h2 );
		v1 = cubic_root( (-yN + v)/(2.0*a) );
		v2 = cubic_root( (-yN - v)/(2.0*a) );

		roots[0] = xN + v1 + v2;
		// print "Raiz en $r1\n";
		return( 1 );
	}
	else {
		double c, s, theta;

		// print "Tres raices reales distintas\n";

		c = -yN/sqrt( h2 );
		s = sqrt( 1.0 - c*c );
		theta = atan2( s, c )/3.0;
		// print "Theta = $theta\n";
		delta = 2.0*sqrt( delta2 );
	
		roots[0] = xN + delta*cos( theta );
		roots[1] = xN + delta*cos( theta + 2.09439510266 );
		roots[2] = xN + delta*cos( theta + 4.18879020533 );
		// print "Raices: $r1 $r2 $r3\n";

		return( 3 );
	}
}

/**
  Calcula los eugenvalores de una matriz simétrica positiva definida A
**/

int eigenValues( double A[][3], double *roots )
{
/**
                                     2                    2              2
(%o5) - s00 s11 s22 + l s11 s22 + s01  s22 + l s00 s22 - l  s22 + s00 s12
        2                      2                    2            2        2
 - l s12  - 2 s01 s02 s12 + s02  s11 + l s00 s11 - l  s11 - l s02  - l s01
    2        3
 - l  s00 + l  = 0
**/

	double b, c, d;

	b = -A[0][0] - A[1][1] - A[2][2];
	c = A[1][1]*A[2][2] + A[0][0]*A[2][2] - A[1][2]*A[1][2] + A[0][0]*A[1][1] - A[0][2]*A[0][2] - A[0][1]*A[0][1]; 
	d = A[0][1]*A[0][1]*A[2][2] + A[0][0]*A[1][2]*A[1][2] - 2.0*A[0][1]*A[0][2]*A[1][2] + A[0][2]*A[0][2]*A[1][1] - A[0][0]*A[1][1]*A[2][2];

	if( cubic( 1.0, b, c, d, roots ) == 3 ) {
		return 3;
	}
	else{
		printf("Esto no debería pasar\n");
		return 0;
	}
}

/* 
   Realiza C = A * B
*/
void multiplicaAB(double A[][3], double B[][3], double C[][3])
{
	int i, j;
	for (i=0; i<3;i++)
		for (j=0;j<3;j++)
			C[i][j] = A[i][0]*B[0][j] + A[i][1]*B[1][j] + A[i][2]*B[2][j];
}

/* 
   Realiza C = A^T * B
*/
void multiplicaATB(double A[][3], double B[][3], double C[][3])
{
	int i, j;
	for (i=0; i<3;i++)
		for (j=0;j<3;j++)
			C[i][j] = A[0][i]*B[0][j] + A[1][i]*B[1][j] + A[2][i]*B[2][j];
}

/* 
   Realiza C = A * B^T
*/
void multiplicaABT(double A[][3], double B[][3], double C[][3])
{
	int i, j;
	for (i=0; i<3;i++)
		for (j=0;j<3;j++)
			C[i][j] = A[i][0]*B[j][0] + A[i][1]*B[j][1] + A[i][2]*B[j][2];
}

int main( )
{
/**
A = [ 1 0 0;
      0 1 2;
      1 1 1 ];
**/
	double A[3][3] = {{1.0, 0.0, 0.0}, {0.0, 1.0, 2.0}, {1.0, 1.0, 1.0} };
	double S[3][3];
	double R[3][3], U[3][3], V[3][3];
	double roots[3];
	int i, j;


	multiplicaATB( A, A, S );
	dsyevv3( S, V, roots );
	// Aqui tengo S = V E V^T

	// printf("%lf %lf %lf\n\n", roots[0], roots[1], roots[2] );

/**
E2 = E;
for i=1:3
    E2(i,i) = 1.0/sqrt( E2(i,i) )
end

U2 = A*V1*E2
R2 = U2*V1'
**/

	// La descomposición en valores singulares de A:
	// A = U D V^T
	// Calculo ahora U = A V D^{-1]
	// D = sqrt(E)
	// D^{-1} :
	for( i=0; i<3; i++ ) 
		roots[i] = 1.0/sqrt(roots[i]);

	// U' = A*V
	multiplicaAB( A, V, U );
		
	// U = U'*D^{-1}
	for( i=0; i<3; i++ ) 
		for( j=0; j<3; j++ ) 
			U[i][j] *= roots[j]; 

	/**
	printf("U:\n");
	for( i=0; i<3; i++ ) {
		for( j=0; j<3; j++ ) {
			printf("%lf ", U[i][j] );
		}
		printf("\n");
	}
	**/
	
	// Finalmente R = U*V^T
	multiplicaABT( U, V, R );
	// 

	for( i=0; i<3; i++ ) {
		for( j=0; j<3; j++ ) {
			printf("%lf ", R[i][j] );
		}
		printf("\n");
	}

	return 0;
}
